function [Ax, Ay, Gx, Gy, Bx, By, vect_temps, delta_, Dtheta_, deltaF_, deltaR_, DA_, Ff_, Fr_, Ffmax_, Frmax_, Vu, Vbeta] = Simulation(scenario, T_simu, Parav, vitesse, braquage, tau)
    %% Caractéristiques véhicule  
    Parav.L = Parav.Lr + Parav.Lf;

    %% Paramètre de simulation
    Paras.tf = T_simu;
    Paras.dt = 1/100;
    Paras.Temps_ = (0:Paras.dt:Paras.tf)';
    vect_temps = Paras.Temps_;
    Paras.Lt = length(Paras.Temps_);
    Paras.AngleBraquage = braquage;
    Tc_beta = T_simu/5; % Temps pour atteindre l'angle de braquage maximum en secondes
    Tc_u = T_simu/2; % Temps pour atteindre la vitesse maximale en secondes
    Tc_freinage = T_simu/10;
    
    %% Variable d'entrée
    u_ = vitesse/3.6 * ones(Paras.Lt, 1);
    beta_ = deg2rad(Paras.AngleBraquage)*ones(Paras.Lt,1);
    
    %Défintion d'une vitesse limite
    ulim = 5/3.6 ;
    %% Initialisation des variables
    Vbeta = zeros(Paras.Lt, 1);
    Vu = zeros(Paras.Lt, 1);
    
    switch scenario
        case 1 
            % Initialisation des variables
            Vbeta = zeros(size(Paras.Temps_));
            Vu = zeros(size(Paras.Temps_));

            % Simulation du freinage brusque dans un virage
            for i = 1:Paras.Lt
                Vbeta(i) = beta_(i) * (Paras.Temps_(i) / Tc_beta) / sqrt(1 + (Paras.Temps_(i) / Tc_beta)^2);
            % Supposons que le freinage brusque est déclenché à un certain moment (par exemple, à mi-chemin)
                if Paras.Temps_(i) <= T_simu / 2
                    Vu(i) = u_(i) * (Paras.Temps_(i) / Tc_freinage) / sqrt(1 + (Paras.Temps_(i) / Tc_freinage)^2);
                else
                    decay_factor = exp(-(Paras.Temps_(i) - T_simu / 2) * tau);
                    Vu(i) = u_(i) * decay_factor * (Paras.Temps_(i) / Tc_freinage) / sqrt(1 + (Paras.Temps_(i) / Tc_freinage)^2);
                end
            end
        case 2 
                for i = 1:Paras.Lt
                    Vbeta(i) = beta_(i) * (Paras.Temps_(i) / Tc_beta) / sqrt(1 + (Paras.Temps_(i) / Tc_beta)^2);
                    Vu(i) = u_(i) * (Paras.Temps_(i) / Tc_u) / sqrt(1 + (Paras.Temps_(i) / Tc_u)^2);
                end
        case 3
            vitesse_ms = vitesse/3.6;
                for i = 1:Paras.Lt
                    obstacle_detection_time = 2.0;
                    arc_duration = 8.0;
                    if Paras.Temps_(i) <= obstacle_detection_time
                        Vu(i) = vitesse_ms;
                        Vbeta(i) = 0;
                    elseif Paras.Temps_(i) <= obstacle_detection_time + arc_duration
                        Vu(i) = vitesse_ms;
                        t = (Paras.Temps_(i) - obstacle_detection_time) / arc_duration;
                        % Utilisez une fonction gaussienne pour obtenir une forme de cloche
                        Vbeta(i) = 0.1 * exp(-(t - 0.5)^2 / (2 * 0.1^2));
                    else
                        % Après l'arc, maintenez la valeur à 0
                        Vu(i) = vitesse_ms;
                        Vbeta(i) = 0;
                    end
                end
    end

            %% Initialisation des variables
        theta_0 = 0;
        Dtheta_0 = 0;
        D2theta_0 = 0;
        delta_0 = (Parav.Lr / (Parav.Lr + Parav.Lf))*Vbeta(1);
        Ddelta_0 = 0;
        deltaR_0 = 0;
        deltaF_0 = 0;
        deltaF_max = 5;
        deltaR_max = 5;
        da_0 = abs(deltaF_0 - deltaR_0);
        % Initialisation des variables de position
        Bx_0 = 0;
        By_0 = 0;
        Gx_0 = Bx_0 + Parav.Lr * cos(theta_0 + delta_0);
        Gy_0 = By_0 + Parav.Lr * sin(theta_0 + delta_0);
        Ax_0 = Bx_0 + (Parav.Lr + Parav.Lf) * cos(theta_0 + beta_(1) + delta_0);
        Ay_0 = By_0 + (Parav.Lr + Parav.Lf) * sin(theta_0 + beta_(1) + delta_0);
        % Initialisation des variables de vitesse en A, B et G selon x et y
        VBx_0 = Vu(1);
        VBy_0 = 0;
        VB_0 = Vu(1); % Norme de la vitesse sur la roue arrière
        VGx_0 = 0;
        VGy_0 = 0;
        VAx_0 = 0;
        VAy_0 = 0;
        % Initialisation des variables d'accélération en A, B et G selon x et y
        ABx_0 = 0;
        ABy_0 = 0;
        AGx_0 = 0;
        AGy_0 = 0;
        AG_0 = 0;
        AAx_0 = 0;
        AAy_0 = 0;
        %% Initialisation des variables d'états
        theta_ = zeros(Paras.Lt,1);
        Dtheta_ = zeros(Paras.Lt,1);
        D2theta_ = zeros(Paras.Lt,1);
        theta_(1) = rad2deg(theta_0);
        Dtheta_(1) = rad2deg(Dtheta_0);
        D2theta_(1) = rad2deg(D2theta_0);
        delta_ = zeros(Paras.Lt,1);
        Ddelta_ = zeros(Paras.Lt,1);
        deltaR_ = zeros(Paras.Lt,1);
        deltaF_ = zeros(Paras.Lt,1);
        DA_ =  zeros(Paras.Lt,1);
        Ddelta_(1) = rad2deg(Ddelta_0);
        delta_(1) = rad2deg(delta_0);
        DA_(1) = da_0;
        Ff_ = zeros(Paras.Lt, 1);
        Fr_ = zeros(Paras.Lt, 1);
        Ffmax_ = zeros(Paras.Lt, 1);
        Frmax_ = zeros(Paras.Lt, 1);
        R = zeros(Paras.Lt, 1);
        Bx = zeros(Paras.Lt, 1);
        By = zeros(Paras.Lt, 1);
        Gx = zeros(Paras.Lt, 1);
        Gy = zeros(Paras.Lt, 1);
        Ax = zeros(Paras.Lt, 1);
        Ay = zeros(Paras.Lt, 1);
        Bx(1) = Bx_0;
        By(1) = By_0;
        Gx(1) = Gx_0;
        Gy(1) = Gy_0;
        Ax(1) = Ax_0;
        Ay(1) = Ay_0;
        VBx = zeros(Paras.Lt, 1);
        VBy = zeros(Paras.Lt, 1);
        VGx = zeros(Paras.Lt, 1);
        VGy = zeros(Paras.Lt, 1);
        VAx = zeros(Paras.Lt, 1);
        VAy = zeros(Paras.Lt, 1);
        VBx(1) = VBx_0 * 3.6; 
        VBy(1) = VBy_0 * 3.6;
        VGx(1) = VGx_0 * 3.6;
        VGy(1) = VGy_0 * 3.6;
        VAx(1) = VAx_0 * 3.6;
        VAy(1) = VAy_0 * 3.6;
        ABx = zeros(Paras.Lt, 1);
        ABy = zeros(Paras.Lt, 1);
        AGx = zeros(Paras.Lt, 1);
        AGy = zeros(Paras.Lt, 1);
        AAx = zeros(Paras.Lt, 1);
        AAy = zeros(Paras.Lt, 1);
%% Boucle de simulation
for i = 2 : Paras.Lt
if Vu(i) <= ulim
% Calcul des angles de lacet et de dérive globaux
Dtheta_1 = Vu(i) * tan(Vbeta(i)) / (Parav.Lr + Parav.Lf);
theta_1 = theta_0 + Paras.dt * Dtheta_1;
delta_1 = (Parav.Lr / (Parav.Lr + Parav.Lf)) * Vbeta(i);
Dtheta_(i) = Dtheta_1 * 180 / pi;
theta_(i) = theta_1 * 180 / pi;
delta_(i) = delta_1 * 180 / pi;
% Calcul des vitesses
VBx_1 = Vu(i);
VBy_1 = 0;
VGx_1 = Vu(i) / cos(delta_1);
VGy_1 = Parav.Lr * Dtheta_1 / sin(delta_1);
VAx_1 = Vu(i) / cos(Vbeta(i));
VAy_1 = (Parav.Lr + Parav.Lf) * Dtheta_1 / sin(Vbeta(i));
VBx(i) = VBx_1 * 3.6;
VBy(i) = VBy_1 * 3.6;
VGx(i) = VGx_1 * 3.6;
VGy(i) = VGy_1 * 3.6;
VAx(i) = VAx_1 * 3.6;
VAy(i) = VAy_1 * 3.6;
% Calcul du rayon de courbure
R_1 = Vu(i) / Dtheta_1;
R(i) = R_1;
R_0 = R_1;
% Calcul des positions
Bx_1 = Bx_0 + VBx_1 * cos(theta_1 + Vbeta(i)) * Paras.dt;
By_1 = By_0 + VBx_1 * sin(theta_1 + Vbeta(i)) * Paras.dt;
Gx_1 = Gx_0 + VGx_1 * cos(theta_1 + Vbeta(i)) * Paras.dt;
Gy_1 = Gy_0 + VGy_1 * sin(theta_1 + Vbeta(i)) * Paras.dt;
Ax_1 = Ax_0 + VAx_1 * cos(theta_1 + Vbeta(i) + deltaF_0) * Paras.dt;
Ay_1 = Ay_0 + VAy_1 * sin(theta_1 + Vbeta(i) + deltaF_0) * Paras.dt;
Bx(i) = Bx_1;
By(i) = By_1;
Gx(i) = Gx_1;
Gy(i) = Gy_1;
Ax(i) = Ax_1;
Ay(i) = Ay_1;
% Calcul des accélérations
ABx_1 = (Vu(i) - Vu(i-1)) / Paras.dt;
ABy_1 = Vu(i) * Dtheta_1;
AGx_1 = -Parav.Lr * Dtheta_1^2 / sin(delta_1);
AGy_1 = Vu(i) * Dtheta_1 / cos(delta_1);
AAx_1 = -(Parav.Lr + Parav.Lf) * Dtheta_1^2 / sin(Vbeta(i));
AAy_1 = Vu(i) * Dtheta_1 / cos(Vbeta(i));
ABx(i) = ABx_1 / 9.81;
ABy(i) = ABy_1 / 9.81;
AGx(i) = AGx_1 / 9.81;
AGy(i) = AGy_1 / 9.81;
AAx(i) = AAx_1 / 9.81;
AAy(i) = AAy_1 / 9.81;
% Mise à jour des variables
Dtheta_0 = Dtheta_1;
theta_0 = theta_1;
delta_0 = delta_1;
Bx_0 = Bx_1;
By_0 = By_1;
Gx_0 = Gx_1;
Gy_0 = Gy_1;
Ax_0 = Ax_1;
Ay_0 = Ay_1;
VBx_0 = VBx_1;
VBy_0 = VBy_1;
VGx_0 = VGx_1;
VGy_0 = VGy_1;
VAx_0 = VAx_1;
VAy_0 = VAy_1;
ABx_0 = ABx_1;
ABy_0 = ABy_1;
AGx_0 = AGx_1;
AGy_0 = AGy_1;
AAx_0 = AAx_1;
AAy_0 = AAy_1;
else
%Calcul des coeffecients de la matrice
a_11 = -(Parav.Cf*Parav.Lf^2 + Parav.Cr*Parav.Lr^2)/(Parav.Iz*Vu(i));
a_12 = (Parav.Cr*Parav.Lr-Parav.Cf*Parav.Lf)/Parav.Iz;
a_21 = ((Parav.Lr*Parav.Cr - Parav.Lf*Parav.Cf)/((Vu(i)^2)*Parav.m))-1;
a_22 = -((Parav.Cr+Parav.Cf)/(Parav.m*Vu(i)));
b_11 = (Parav.Lf*Parav.Cf)/Parav.Iz;
b_21 = Parav.Cf/(Parav.m*Vu(i));
%% Calcul des angles
D2theta_1 = (a_11)*Dtheta_0 + (a_12)*delta_0 + (b_11)*Vbeta(i);
Dtheta_1 = Dtheta_0 + Paras.dt*D2theta_1;
theta_1 = theta_0 + Paras.dt*Dtheta_1;
Ddelta_1 = (a_21)* Dtheta_0 + (a_22)*delta_0 + (b_21)*Vbeta(i);
delta_1 = delta_0 + Paras.dt*Ddelta_1;
deltaF_1 = delta_1 - Vbeta(i) + (Parav.Lf*Dtheta_1/Vu(i));
deltaR_1 = delta_1 - (Parav.Lr*Dtheta_1/Vu(i));
Ff_1 = -Parav.Cf * deltaF_1;
Fr_1 = -Parav.Cr * deltaR_1;
if abs(deltaF_1) > (deltaF_max)* pi/180
    Ffmax_1 = - Parav.Cf * deltaF_max * sign(deltaF_1) * pi/180;
else
    Ffmax_1 = Ff_1;
end
if abs(deltaR_1) > (deltaR_max)* pi/180
    Frmax_1 = - Parav.Cr * deltaR_max * sign(deltaR_1) * pi/180;
else
    Frmax_1 = Fr_1;
end
% Calcul des positions des roues Avant (A) et Arrière (B) et du Centre de gravité (G)
Bx_1 = Bx_0 + Vu(i) * Paras.dt * cos(theta_1 + deltaR_1);
By_1 = By_0 + Vu(i) * Paras.dt * sin(theta_1 + deltaR_1);
% Calcul des positions en A et G
Gx_1 = Bx_0 + Parav.Lr * cos(theta_1 + delta_1);
Gy_1 = By_0 + Parav.Lr * sin(theta_1 + delta_1);
Ax_1 = Bx_0 + (Parav.Lf + Parav.Lr) * cos(theta_1 - deltaF_1);
Ay_1 = By_0 + (Parav.Lf + Parav.Lr) * sin(theta_1 - deltaF_1);
% Calcul des vitesses en A, B et G
VGx_1 = Vu(i) * cos(delta_1);
VGy_1 = Vu(i) * sin(delta_1);
VAx_1 = (Vu(i) * cos(delta_1)) / (cos(deltaF_1 + Vbeta(i)));
VAy_1 = (Vu(i) * sin(delta_1) + Parav.Lf * Dtheta_1) / (sin(deltaF_1 + Vbeta(i)));
VBx_1 = (Vu(i) * cos(delta_1)) / (cos(deltaR_1));
VBy_1 = (Vu(i) * sin(delta_1) - Parav.Lr * Dtheta_1) / (sin(deltaR_1));
% Calcul des accélérations en A, B et G
AGx_1 = (-Ff_1 * sin(Vbeta(i))) / Parav.m;
AGy_1 = (Fr_1 + Ff_1 * cos(Vbeta(i))) / Parav.m;
AG_1 = sqrt(AGx_1^2 + AGy_1^2);
AAx_1 = (VAx_1 - VAx_0) / Paras.dt;
AAy_1 = (VAy_1 - VAy_0) / Paras.dt;
ABx_1 = (VBx_1 - VBx_0) / Paras.dt;
ABy_1 = (VBy_1 - VBy_0) / Paras.dt;
% Calcul du rayon de courbure
VB_1 = sqrt(VBx_1^2 + VBy_1^2);
R_1 = VB_1 / Dtheta_1;
% Calcul de la différence entre les angles de dérive avant et arrière
da_1 = abs(deltaF_1) - abs(deltaR_1);
% Incorporation dans les vecteurs de sortie
delta_(i) = rad2deg(delta_1);
Ddelta_(i) = rad2deg(Ddelta_1);
deltaR_(i) = rad2deg(deltaR_1);
deltaF_(i) = rad2deg(deltaF_1);
theta_(i) = rad2deg(theta_1);
Dtheta_(i) = rad2deg(Dtheta_1);
D2theta_(i) = rad2deg(D2theta_1);
DA_(i) = da_1 * 180 / pi;
Ff_(i) = Ff_1;
Fr_(i) = Fr_1;
Ffmax_(i) = Ffmax_1;
Frmax_(i) = Frmax_1;
Bx(i) = Bx_1;
By(i) = By_1;
Gx(i) = Gx_1;
Gy(i) = Gy_1;
Ax(i) = Ax_1;
Ay(i) = Ay_1;
VBx(i) = VBx_1 * 3.6;
VBy(i) = VBy_1 * 3.6;
VGx(i) = VGx_1 * 3.6;
VGy(i) = VGy_1 * 3.6;
VAx(i) = VAx_1 * 3.6;
VAy(i) = VAy_1 * 3.6;
ABx(i) = ABx_1 / 9.81;
ABy(i) = ABy_1 / 9.81;
AGx(i) = AGx_1 / 9.81;
AGy(i) = AGy_1 / 9.81;
AAx(i) = AAx_1 / 9.81;
AAy(i) = AAy_1 / 9.81;
R(i) = R_1;
% Mise à jour des variables
D2theta_0 = D2theta_1;
Dtheta_0 = Dtheta_1;
theta_0 = theta_1;
delta_0 = delta_1;
deltaF_0 = deltaF_1;
deltaR_0 = deltaR_1;
da_0 = da_1;
Ff_0 = Ff_1;
Fr_0 = Fr_1;
Frmax_0 = Frmax_1;
Ffmax_0 = Ffmax_1;
Ax_0 = Ax_1;
Ay_0 = Ay_1;
Bx_0 = Bx_1 ;
By_0 = By_1 ;
Gx_0 = Gx_1;
Gy_0 = Gy_1;
R_0 = R_1;
VBx_0 = VBx_1;
VBy_0 = VBy_1;
VB_0 = VB_1;
VGx_0 = VGx_1;
VGy_0 = VGy_1;
VAx_0 = VAx_1;
VAy_0 = VAy_1;
ABx_0 = ABx_1;
ABy_0 = ABy_1;
AGx_0 = AGx_1;
AGy_0 = AGy_1;
AG_0 = AG_1;
AAx_0 = AAx_1;
AAy_0 = AAy_1;
end
end
end